#version 120

uniform float centerDepthSmooth;
uniform float far;
uniform float frameTimeCounter;
uniform int heldBlockLightValue;
uniform int heldItemId;
uniform ivec2 eyeBrightnessSmooth;
uniform mat4 gbufferProjection;
uniform mat4 gbufferProjectionInverse;

varying float eyeAdjust; //How much brighter to make the world

varying vec2 texcoord;

#define BLUR_ENABLED //Is blur enabled at all?
//#define DOF_BLUR //Blurs things that are at a different distance than whatever's in the center of your screen

#define DYNAMIC_LIGHTS //Holding blocks that emit light will light up their surroundings
#define EYE_ADJUST //Allows your eyes to "adjust" to darkness

varying vec3 BLOCK_LIGHT_COLOR; //Color of block light. Gets yellow-er if you stay away from light-emitting blocks for a while.
varying vec4 HELD_LIGHT_COLOR; //Color of held light source. Alpha = brightness.

#if defined(BLUR_ENABLED) && defined(DOF_BLUR)
	varying float dofDistance; //Un-projected centerDepthSmooth
#endif

uniform sampler2D noisetex;
const int noiseTextureResolution = 64;
const float invNoiseRes = 1.0 / float(noiseTextureResolution);

#ifdef DYNAMIC_LIGHTS
	varying float heldLightDistModifier; //Zoom in with optifine to increase range of held lights.

	float flicker() {
		float n = texture2D(noisetex, frameTimeCounter * vec2(16.7825, 15.4192) * invNoiseRes).r - 0.5;
		return n * n * n * 12.0;
	}

	vec4 calcHeldLightColor(float light, int id) { //rgb = color, a = brightness
		if   (light == 0.0) return vec4(0.0); //not holding a light source
		else if (id == 50 ) return vec4(1.0,  0.6,  0.3, light + flicker()); //torches
		else if (id == 89 ) return vec4(1.0,  0.6,  0.1, light            ); //glowstone
		else if (id == 169) return vec4(0.6,  0.8,  0.6, light            ); //sea lanterns
		else if (id == 198) return vec4(0.75, 0.55, 0.8, light            ); //end rods
		else if (id == 76 ) return vec4(1.0,  0.3,  0.1, light + flicker()); //redstone torches
		else if (id == 91 ) return vec4(1.0,  0.6,  0.3, light + flicker()); //jack-o-lanterns
		else                return vec4(0.8,  0.65, 0.5, light            ); //everything else
	}
#endif

void main() {
	gl_Position = ftransform();
	texcoord = (gl_TextureMatrix[0] * gl_MultiTexCoord0).xy;

	#ifdef EYE_ADJUST
		eyeAdjust = 2.5 - eyeBrightnessSmooth.x / 240.0;
	#else
		eyeAdjust = 1.5;
	#endif

	#if defined(BLUR_ENABLED) && defined(DOF_BLUR)
		vec4 v = gbufferProjectionInverse * vec4(0.0, 0.0, centerDepthSmooth * 2.0 - 1.0, 1.0);
		dofDistance = -v.z / v.w;
	#endif

	BLOCK_LIGHT_COLOR = mix(vec3(1.5, 0.75, 0.25), vec3(1.4, 1.1, 0.8), eyeBrightnessSmooth.x / 240.0);

	#ifdef DYNAMIC_LIGHTS
		HELD_LIGHT_COLOR = calcHeldLightColor(float(heldBlockLightValue),  heldItemId);
		heldLightDistModifier = far / sqrt(gbufferProjection[1][1]); //held lights get more powerful when zooming in, akin to holding the light out in front of you and pointing it at something.
	#else
		HELD_LIGHT_COLOR = vec4(0.0);
	#endif
}