#version 120

attribute vec3 mc_Entity;

uniform float far;
uniform float frameTimeCounter;
uniform int heldBlockLightValue;
uniform int heldItemId;
uniform ivec2 eyeBrightnessSmooth;
uniform mat4 gbufferModelViewInverse;
uniform mat4 gbufferProjection;

varying float mcentity; //ID data of block currently being rendered.
varying vec2 lmcoord;
varying vec2 texcoord;
varying vec3 normal;
varying vec3 pos;
varying vec4 glcolor;

varying vec3 BLOCK_LIGHT_COLOR; //Color of block light. Gets yellow-er if you stay away from light-emitting blocks for a while.
varying vec4 HELD_LIGHT_COLOR; //Color of held light source. Alpha = brightness.

#define DYNAMIC_LIGHTS //Holding blocks that emit light will light up their surroundings
//#define ID_FIX //Enable this if "effect blocks" are lacking their respective effects. For example if grass doesn't sway back and forth, or if water doesn't refract things behind it.

uniform sampler2D noisetex;
const int noiseTextureResolution = 64;
const float invNoiseRes = 1.0 / float(noiseTextureResolution);

#ifdef DYNAMIC_LIGHTS
	varying float heldLightDistModifier; //Zoom in with optifine to increase range of held lights.

	float flicker() {
		float n = texture2D(noisetex, frameTimeCounter * vec2(16.7825, 15.4192) * invNoiseRes).r - 0.5;
		return n * n * n * 12.0;
	}

	vec4 calcHeldLightColor(float light, int id) { //rgb = color, a = brightness
		if   (light == 0.0) return vec4(0.0); //not holding a light source
		else if (id == 50 ) return vec4(1.0,  0.6,  0.3, light + flicker()); //torches
		else if (id == 89 ) return vec4(1.0,  0.6,  0.1, light            ); //glowstone
		else if (id == 169) return vec4(0.6,  0.8,  0.6, light            ); //sea lanterns
		else if (id == 198) return vec4(0.75, 0.55, 0.8, light            ); //end rods
		else if (id == 76 ) return vec4(1.0,  0.3,  0.1, light + flicker()); //redstone torches
		else if (id == 91 ) return vec4(1.0,  0.6,  0.3, light + flicker()); //jack-o-lanterns
		else                return vec4(0.8,  0.65, 0.5, light            ); //everything else
	}
#endif

void main() {
	texcoord = (gl_TextureMatrix[0] * gl_MultiTexCoord0).xy;
	lmcoord = (gl_TextureMatrix[1] * gl_MultiTexCoord1).xy;
	glcolor = gl_Color;
	gl_Position = ftransform();
	pos = (gbufferModelViewInverse * (gl_ModelViewMatrix * gl_Vertex)).xyz;
	pos.y -= 1.6;
	normal = normalize(gl_Normal) * 0.5 + 0.5;

	//Using IDs above 10000 to represent all blocks that I care about
	//if the ID is less than 10000, then I don't need to do extra logic to see if it has special effects.
	#ifndef ID_FIX
		if (mc_Entity.x > 10000.0) {
			int id = int(mc_Entity.x) - 10000;
	#else
			int id = int(mc_Entity.x);
	#endif
			#ifndef ID_FIX
				if      (id == 9)  mcentity = 1.1; //water
			#else
				if      (id == 8 || id == 9)  mcentity = 1.1; //water
			#endif
			#ifndef ID_FIX
				else if (id == 95) mcentity = 2.1; //stained glass
			#else
				else if (id == 95 || id == 160) mcentity = 2.1; //stained glass
			#endif
			else if (id == 79) mcentity = 3.1; //ice
			else mcentity = 0.1;
	#ifndef ID_FIX
		}
		else mcentity = 0.1;
	#endif

	BLOCK_LIGHT_COLOR = mix(vec3(1.5, 0.75, 0.25), vec3(1.4, 1.1, 0.8), eyeBrightnessSmooth.x / 240.0);

	#ifdef DYNAMIC_LIGHTS
		HELD_LIGHT_COLOR = calcHeldLightColor(float(heldBlockLightValue),  heldItemId);
		heldLightDistModifier = far / sqrt(gbufferProjection[1][1]); //held lights get more powerful when zooming in, akin to holding the light out in front of you and pointing it at something.
	#else
		HELD_LIGHT_COLOR = vec4(0.0);
	#endif
}