#version 120

uniform float rainStrength;
uniform int worldTime;
uniform vec3 sunPosition;

varying vec2 lmcoord;
varying vec2 texcoord;
varying vec3 pos;
varying vec4 glcolor;

void main() {
	texcoord = (gl_TextureMatrix[0] * gl_MultiTexCoord0).xy;
	lmcoord = (gl_TextureMatrix[1] * gl_MultiTexCoord1).xy;
	gl_Position = ftransform();
	pos = (gl_ModelViewMatrix * gl_Vertex).xyz;
	glcolor = gl_Color;

	float adjustedTime = abs(mod(worldTime / 1000.0 + 6.0, 24.0) - 12.0);
	float night = clamp(adjustedTime - 6.7, 0.0, 1.0);

	vec3 sunPosNorm = normalize(sunPosition);

	vec3 normal = normalize(gl_NormalMatrix * gl_Normal);
	float glmult = 0.0;
	if (night < 0.999) glmult += dot( sunPosNorm, normal) * (1.0 - night);
	if (night > 0.001) glmult += dot(-sunPosNorm, normal) * night;
	glmult = glmult * 0.375 + 0.625; //0.25 - 1.0
	glmult = mix(glmult, 1.0, rainStrength * 0.5); //less shading during rain
	glmult = mix(1.0, glmult, lmcoord.y * 0.66666666 + 0.33333333); //0.5 - 1.0 in darkness
	glmult = mix(glmult, 1.0, lmcoord.x * lmcoord.x); //increase brightness when block light is high
	glcolor.rgb *= glmult;
}