#version 120

uniform int moonPhase;
uniform int worldTime;
uniform mat4 gbufferModelView;

varying float adjustedTime; //More useful number than worldTime.
varying float day; //1.0 during the day, 0.0 at night.
varying float night; //1.0 at night, 0.0 during the day.
varying float sunset; //1.0 at sunset (when day and night are both < 1.0).
varying float phase; //More useful number than moonPhase.

varying vec3 sunPosNorm;
varying vec3 upPosNorm;

varying vec4 starData; //rgb = star color, a = flag for weather or not this pixel is a star.

#define SUN_POSITION_FIX //Enable this if the horizon "splits" at sunset when rapidly rotating your camera.

#ifdef SUN_POSITION_FIX
	const float sunPathRotation = 30.0; //Angle that the sun/moon rotate at [-45.0 -40.0 -35.0 -30.0 -25.0 -20.0 -15.0 -10.0 -5.0 0.0 5.0 10.0 15.0 20.0 25.0 30.0 35.0 40.0 45.0]
	const vec2 sunRotationData = vec2(cos(sunPathRotation * 0.01745329251994), -sin(sunPathRotation * 0.01745329251994)); //Used for manually calculating the sun's position, since the sunPosition uniform is inaccurate in the skybasic stage.
#else
	uniform vec3 sunPosition;
	uniform vec3 upPosition;
#endif

void main() {
	gl_Position = ftransform();

	adjustedTime = abs(mod(worldTime / 1000.0 + 6.0, 24.0) - 12.0);
	day = clamp(6.7 - adjustedTime, 0.0, 1.0);
	night = clamp(adjustedTime - 6.7, 0.0, 1.0);
	sunset = 1.0 - day - night;
	phase = abs(moonPhase / 4.0 - 1.0);

	starData = vec4(gl_Color.rgb, float(gl_Color.r == gl_Color.g && gl_Color.g == gl_Color.b && gl_Color.r > 0.0));

	#ifdef SUN_POSITION_FIX
		//minecraft's native calculateCelestialAngle() function, optimized and ported to GLSL.
		float ang = fract(worldTime / 24000.0 - 0.25);
		ang = (ang + (cos(ang * 3.14159265358979) * -0.5 + 0.5 - ang) / 3.0) * 6.28318530717959; //0-2pi, rolls over from 2pi to 0 at noon.

		sunPosNorm = normalize((gbufferModelView * vec4(vec3(-sin(ang), cos(ang) * sunRotationData) * 100.0, 1.0)).xyz);
		upPosNorm = gbufferModelView[1].xyz;
	#else
		sunPosNorm = normalize(sunPosition);
		upPosNorm = normalize(upPosition);
	#endif
}