#version 120

uniform float aspectRatio;
uniform float blindness;
uniform float far;
uniform float frameTimeCounter;
uniform float nightVision;
uniform float screenBrightness;
uniform float viewHeight;
uniform float viewWidth;
uniform int isEyeInWater;
uniform ivec2 eyeBrightness;
uniform ivec2 eyeBrightnessSmooth;
uniform mat4 gbufferModelView;
uniform mat4 gbufferModelViewInverse;
uniform mat4 gbufferProjectionInverse;
uniform sampler2D composite;
uniform sampler2D depthtex0;
uniform sampler2D gaux2;
uniform sampler2D gaux4;
uniform sampler2D gcolor;
uniform sampler2D gnormal;
uniform vec3 cameraPosition;
        vec3 eyePosition = cameraPosition + gbufferModelViewInverse[3].xyz; //because cameraPosition isn't actually the position of the camera -_-
uniform vec3 fogColor;
uniform vec3 skyColor;

varying float eyeAdjust; //How much brighter to make the world

varying vec2 texcoord;

#define BLUR_ENABLED //Is blur enabled at all?
#define UNDERWATER_BLUR //Blurs the world while underwater
#define WATER_BLUR //Blurs things behind water
#define GLASS_BLUR //Blurs things behind stained glass
#define ICE_BLUR //Blurs things behind ice
//#define DOF_BLUR //Blurs things that are at a different distance than whatever's in the center of your screen

#define FOG //It's fog, what more is there to say here?
#define UNDERWATER_FOG //Applies fog to water
#define DYNAMIC_LIGHTS //Holding blocks that emit light will light up their surroundings
//#define VANILLA_LIGHTMAP //Uses vanilla light colors instead of custom ones. Requires optifine 1.12.2 HD_U_D1 or later!
#define VIGNETTE //Reduces the brightness of dynamic light around edges the of your screen
//#define CROSS_PROCESS //Opposite of desaturation, makes everything more vibrant and saturated.
#define WATER_REFRACT //Adds "waves" to water that distort things behind them
#define REFLECT //Reflects the sun/sky onto reflective surfaces. Does not add reflections of terrain!
#define ICE_SCATTER //Scatters things behind ice
#define ALT_GLASS //Uses alternate blending method for stained glass which looks more like real stained glass
#define BRIGHT_WATER //Overrides light levels under water to be higher
//#define CUBIC_CHUNKS //Disables black fog/sky colors below Y=0

#if defined(BLUR_ENABLED) && defined(DOF_BLUR)
	varying float dofDistance; //Un-projected centerDepthSmooth
#endif

varying vec4 HELD_LIGHT_COLOR; //Color of held light source. Alpha = brightness.

uniform sampler2D noisetex;
const int noiseTextureResolution = 64;
const float invNoiseRes = 1.0 / float(noiseTextureResolution);

float square(float x)        { return x * x; } //faster than pow().

float interpolateSmooth1(float x) { return x * x * (3.0 - 2.0 * x); }

float fogify(float x, float width) {
	//fast, vaguely bell curve-shaped function with variable width
	return width / (x * x + width);
}

/*
#ifdef BLUR_ENABLED
#endif
#ifdef UNDERWATER_BLUR
#endif
#ifdef WATER_BLUR
#endif
#ifdef GLASS_BLUR
#endif
#ifdef ICE_BLUR
#endif
#ifdef RAIN_BLUR
#endif
#ifdef DOF_BLUR
#endif

Are you happy now, optifine?
*/

//basically the simplified mathematical form of "clr * mix(multiplier, 1.0, clr)".
//the brighter a color was originally, the less of an effect multiplier has on it.
vec3 brightenColor(vec3 clr, float multiplier) {
	return clr * (multiplier - clr * (multiplier - 1.0));
}

const vec2[] goldenOffsets = vec2[](
	vec2( 0.675490294261524, -0.73736887807832 ), //2.39996322972865332
	vec2(-0.996171040864828,  0.087425724716963), //4.79992645945731
	vec2( 0.793600751291696,  0.608438860978863), //7.19988968918596
	vec2(-0.174181950379306, -0.98471348531543 ), //9.59985291891461
	vec2(-0.53672805262632,   0.843755294812399), //11.9998161486433
	vec2( 0.965715074375778, -0.259604304901489), //14.3997793783719
	vec2(-0.887448429245268, -0.460907024713344), //16.7997426081006
	vec2( 0.343038630874082,  0.939321296324125), //19.1997058378292
	vec2( 0.38155640847493,  -0.924345556137807), //21.5996690675579
	vec2(-0.905734272555614, -0.04619144594037 )  //23.9996322972865
); //sines and cosines of multiples of the golden angle (~2.4 radians)

#ifdef DYNAMIC_LIGHTS
	varying float heldLightDistModifier; //Zoom in with optifine to increase range of held lights.
#endif

vec3 calcMainLightColor(inout float blocklight, inout float skylight, inout float heldlight, in float dist) {
	#ifdef VANILLA_LIGHTMAP
		vec3 lightclr = texture2D(gaux4, vec2(blocklight, skylight)).rgb;
	#endif

	skylight *= skylight;
	blocklight = square(max(blocklight - skylight * 0.5, 0.0));

	#ifndef VANILLA_LIGHTMAP
		vec3 lightclr = vec3(0.0);
		lightclr += mix(vec3(1.0, 0.5, 0.15), vec3(1.0, 0.85, 0.7), eyeBrightnessSmooth.x / 240.0) * blocklight; //blocklight
		lightclr += mix(skyColor, vec3(1.0), skylight) * skylight; //skylight
		lightclr += nightVision * 0.5 + screenBrightness * 0.1;
	#endif

	#ifdef DYNAMIC_LIGHTS
		float d = dist * heldLightDistModifier;
		if (d < HELD_LIGHT_COLOR.a * 2.0) {
			heldlight = max(HELD_LIGHT_COLOR.a / square(d + 3.0) * (HELD_LIGHT_COLOR.a * 2.0 - d) / HELD_LIGHT_COLOR.a - (skylight + blocklight) * 0.75, 0.0);
			#ifdef VIGNETTE
				heldlight *= (1.0 - length(gl_FragCoord.xy / vec2(viewWidth, viewHeight) - 0.5)); //helps reduce the "circle that follows you" effect by making held lights darker towards the edge of your screen
			#endif
			lightclr += HELD_LIGHT_COLOR.rgb * heldlight;
		}
	#endif

	return lightclr;
}

#ifdef WATER_REFRACT
	vec2 waterNoise(vec2 coord, float time) {
		coord *= invNoiseRes;

		vec2 noise = vec2(0.0);
		noise += (texture2D(noisetex, (coord + goldenOffsets[0] * time)      ).rg - 0.5);          //1.0 / 1.0
		noise += (texture2D(noisetex, (coord + goldenOffsets[1] * time) * 1.5).rg - 0.5) * 0.6666; //1.0 / 1.5
		noise += (texture2D(noisetex, (coord + goldenOffsets[2] * time) * 2.0).rg - 0.5) * 0.5;    //1.0 / 2.0
		noise += (texture2D(noisetex, (coord + goldenOffsets[3] * time) * 2.5).rg - 0.5) * 0.4;    //1.0 / 2.5
		noise += (texture2D(noisetex, (coord + goldenOffsets[4] * time) * 3.0).rg - 0.5) * 0.3333; //1.0 / 3.0
		noise += (texture2D(noisetex, (coord + goldenOffsets[5] * time) * 3.5).rg - 0.5) * 0.2857; //1.0 / 3.5
		noise += (texture2D(noisetex, (coord + goldenOffsets[6] * time) * 4.0).rg - 0.5) * 0.25;   //1.0 / 4.0
		return noise;
	}

	vec2 waterNoiseLOD(vec2 coord, float distance) {
		float lod = log2(distance * 0.0625); //level of detail
		float scale = floor(lod);
		coord *= exp2(-scale); //each time the distance doubles, so will the scale factor
		float middle = fract(lod);
		float time = frameTimeCounter * invNoiseRes * 3.0;

		vec2 noise1 = waterNoise(coord, time / max(scale, 1.0));
		vec2 noise2 = waterNoise(coord * 0.5, time / max(scale + 1.0, 1.0));

		return mix(noise1, noise2, interpolateSmooth1(middle));
	}
#endif

#ifdef ICE_SCATTER
	vec3 iceNoise(vec2 coord) {
		coord *= invNoiseRes;

		vec3 noise = vec3(0.0);
		noise += texture2D(noisetex, coord        ).rgb;
		noise += texture2D(noisetex, coord * 0.5  ).rgb;
		noise += texture2D(noisetex, coord * 0.25 ).rgb;
		noise += texture2D(noisetex, coord * 0.125).rgb;
		noise -= 2.0; //0.5 * 4.0
		return noise;
	}

	vec3 iceNoiseLOD(vec2 coord, float distance) {
		float lod = log2(distance); //level of detail
		float scale = exp2(-floor(lod)); //each time the distance doubles, so will the scale factor
		coord *= scale;
		float middle = fract(lod);

		vec3 noise1 = iceNoise(coord      );
		vec3 noise2 = iceNoise(coord * 0.5);

		return mix(noise1, noise2, interpolateSmooth1(middle));
	}
#endif

vec3 calcFogColor(vec3 pos) {
	#ifndef CUBIC_CHUNKS
		if (cameraPosition.y < -gbufferModelViewInverse[3][1]) return vec3(0.0);
	#endif

	return mix(skyColor, fogColor, fogify(max(dot(pos, gbufferModelView[1].xyz), 0.0), 0.0625));
}

vec3 calcUnderwaterFogColor(vec3 color, float dist, float brightness) {
	dist *= far; //use absolute distance, not relative distance

	vec3 multiplier = exp2(-dist * vec3(0.1875, 0.0625, 0.125));
	vec3 scatterColor = (skyColor + fogColor) * vec3(0.0625, 0.125, 0.1875) * (1.0 - multiplier) * brightness;
	return color * multiplier + scatterColor;
}

void main() {
	if (isEyeInWater == 2) { //under lava
		vec2 coord = floor(vec2(texcoord.x, texcoord.y / aspectRatio) * 24.0 + vec2(0.0, frameTimeCounter)) + 0.5; //24.0 is the resolution of the generated lava texture.
		float noise = 0.0;
		noise += (texture2D(noisetex, (coord * 0.25 + vec2(0.0, frameTimeCounter)) * invNoiseRes).r - 0.5);
		noise += (texture2D(noisetex, (coord * 0.5  + vec2(0.0, frameTimeCounter)) * invNoiseRes).r - 0.5) * 0.5;
		noise += (texture2D(noisetex, (coord        + vec2(0.0, frameTimeCounter)) * invNoiseRes).r - 0.5) * 0.25;
		vec3 color = vec3(1.0, 0.5, 0.0) + noise * vec3(0.375, 0.5, 0.5);
		gl_FragData[0] = vec4(color, 1.0);
		return; //don't need to calculate anything else since the lava overlay covers the entire screen.
	}

	vec2 tc = texcoord;

	vec3 oldAux2 = texture2D(gaux2, texcoord).rgb;
	int id = int(oldAux2.b * 10.0 + 0.1);
	vec3 normal = texture2D(gnormal, texcoord).xyz * 2.0 - 1.0;

	vec3 pos = vec3(texcoord, texture2D(depthtex0, texcoord).r);
	bool nothingInFrontOfSky = pos.z == 1.0;
	vec4 v = gbufferProjectionInverse * vec4(pos * 2.0 - 1.0, 1.0);
	pos = v.xyz / v.w;

	float dist = length(pos);
	vec3 posNorm = pos / dist;

	vec3 worldPos = (gbufferModelViewInverse * vec4(pos, 1.0)).xyz + cameraPosition;

	#ifdef REFLECT
		float reflective = 0.0;
	#endif

	float blur = 0.0;

	#if defined(BLUR_ENABLED) && defined(DOF_BLUR)
		blur = interpolateSmooth1(min(abs(dist - dofDistance) / dofDistance, 1.0)) * 4.0;
	#endif

	#if defined(BLUR_ENABLED) && defined(WATER_BLUR)
		float waterBlur = 4.0; //slightly more dynamic than other types of blur
	#endif

	if (id == 1) { //water
		#ifdef REFLECT
			reflective = 0.5;
		#endif

		#ifdef WATER_REFRACT
			vec3 newPos = worldPos;
			ivec2 swizzles;
			float multiplier = 1.0;
			if (abs(normal.y) > 0.1) { //top/bottom surfaces
				if (abs(normal.y) < 0.999) newPos.xz -= normalize(normal.xz) * (frameTimeCounter * 3.0);
				else multiplier = oldAux2.g * 0.75 + 0.25;
				swizzles = ivec2(0, 2);
			}
			else {
				newPos.y += frameTimeCounter * 4.0;
				if (abs(normal.x) < 0.02) swizzles = ivec2(0, 1);
				else swizzles = ivec2(2, 1);
			}

			vec2 offset = waterNoiseLOD(vec2(newPos[swizzles[0]], newPos[swizzles[1]]), dist) * (multiplier * 0.015625);
			normal[swizzles[0]] += offset[0] * 4.0;
			normal[swizzles[1]] += offset[1] * 4.0;

			offset.y *= aspectRatio;

			tc += offset / max(dist * 0.0625, 1.0);
		#endif
	}
	else if (id == 2) { //stained glass
		#ifdef REFLECT
			reflective = 0.25;
		#endif

		#if defined(BLUR_ENABLED) && defined(GLASS_BLUR)
			blur = max(blur, 8.0);
		#endif
	}
	else if (id == 3 || id == 4) { //ice and held ice
		#ifdef REFLECT
			reflective = 0.25;
		#endif

		#if defined(BLUR_ENABLED) && defined(ICE_BLUR)
			blur = max(blur, 4.0);
		#endif

		#ifdef ICE_SCATTER
			vec3 offset;
			if (id == 3) { //normal ice
				vec2 coord = (abs(normal.y) < 0.001 ? vec2(worldPos.x + worldPos.z, worldPos.y) : worldPos.xz);
				offset = iceNoiseLOD(coord * 256.0, dist) * 0.0078125;
			}
			else { //held ice
				vec2 coord = gl_FragCoord.xy + 0.5;
				offset = iceNoise(coord * 0.5) * 0.0078125;
			}

			normal += offset * 8.0;

			offset.y *= aspectRatio;
			vec2 newtc = tc + offset.xy;
			vec3 newnormal = texture2D(gnormal, newtc).xyz * 2.0 - 1.0;
			if (dot(normal, newnormal) > 0.9) tc = newtc; //don't offset on the edges of ice
		#endif
	}

	vec3 aux2 = texture2D(gaux2, tc).rgb;
	if (abs(aux2.b - oldAux2.b) > 0.02) {
		tc = texcoord;
		aux2 = texture2D(gaux2, tc).rgb;
	}

	vec4 c = texture2D(gcolor, tc);
	vec3 color = c.rgb;
	float transparentAlpha = c.a; //using gcolor to store composite's alpha
	vec4 transparent = texture2D(composite, tc); //transparency of closest object to the camera

	#if defined(BLUR_ENABLED) && defined(UNDERWATER_BLUR)
		if (isEyeInWater == 1) blur = 6.0;
	#endif

	dist /= far;

	if (transparentAlpha > 0.001) {
		#ifdef ALT_GLASS
			if (id == 2) {
				vec3 transColor = transparent.rgb / transparentAlpha;
				color *= min(transColor * 2.0, 1.0); //because the default colors are too dark to be used.

				float skylight = aux2.g;
				float blocklight = aux2.r;
				float heldlight = 0.0;

				color += transColor * calcMainLightColor(blocklight, skylight, heldlight, dist) * 0.125 * (1.0 - blindness);
			}
			else {
		#endif
			color = mix(color, transparent.rgb / transparentAlpha, transparentAlpha);
		#ifdef ALT_GLASS
			}
		#endif
	}

	#ifdef REFLECT
		reflective *= aux2.g * aux2.g * (1.0 - blindness);
		vec3 reflectedPos;
		if (isEyeInWater == 0 && reflective > 0.001) { //sky reflections
			vec3 newnormal = normalize((gbufferModelView * vec4(normal, 0.0)).xyz);
			reflectedPos = reflect(posNorm, newnormal);
			vec3 skyclr = calcFogColor(reflectedPos);
			float posDot = dot(-posNorm, newnormal);
			color += skyclr * square(square(1.0 - max(posDot, 0.0))) * reflective;
		}
	#endif

	if (id >= 1) { //everything that I've currently assigned effects to so far needs fog to be done in this stage.
		if (isEyeInWater == 1) {
			#ifdef UNDERWATER_FOG
				float actualEyeBrightness = eyeBrightnessSmooth.y / 240.0;
				#ifdef BRIGHT_WATER
					actualEyeBrightness = actualEyeBrightness * 0.5 + 0.5;
				#endif
				color = calcUnderwaterFogColor(color, dist, actualEyeBrightness) * (1.0 - blindness);
			#endif
		}
		else {
			#ifdef FOG
				float d = dist - 0.2;
				if (d > 0.0) {
					d = fogify(d * exp2(1.5 - worldPos.y * 0.015625), 0.25);
					color = mix(calcFogColor(posNorm) * min(max(aux2.g * 2.0, eyeBrightness.y / 120.0), 1.0) * (1.0 - blindness), color, d);
					#if defined(BLUR_ENABLED) && defined(WATER_BLUR)
						waterBlur *= d;
					#endif
				}
			#endif
		}
	}

	color = min(color, 1.0); //reflections (And possibly other things) can go above maximum brightness

	#if defined(BLUR_ENABLED) && defined(WATER_BLUR)
		if (id == 1 && isEyeInWater == 0) blur += waterBlur;
	#endif

	#ifdef BLUR_ENABLED
		blur /= 256.0;
	#endif

	color = brightenColor(color, eyeAdjust);

/* DRAWBUFFERS:6 */
	gl_FragData[0] = vec4(color, 1.0 - blur); //gcolor
}