#version 120

uniform float blindness;
uniform float far;
uniform float frameTimeCounter;
uniform float nightVision;
uniform float screenBrightness;
uniform float viewHeight;
uniform float viewWidth;
uniform ivec2 eyeBrightnessSmooth;
uniform mat4 gbufferModelViewInverse;
uniform mat4 gbufferProjectionInverse;
uniform sampler2D depthtex1;
uniform sampler2D gaux1;
uniform sampler2D gaux3;
uniform sampler2D gaux4;
#define lightmap gaux4
uniform sampler2D gcolor;
uniform vec3 fogColor;

varying vec2 texcoord;

#define FOG //It's fog, what more is there to say here?
#define DYNAMIC_LIGHTS //Holding blocks that emit light will light up their surroundings
//#define VANILLA_LIGHTMAP //Uses vanilla light colors instead of custom ones. Requires optifine 1.12.2 HD_U_D1 or later!
#define VIGNETTE //Reduces the brightness of dynamic light around edges the of your screen
//#define CROSS_PROCESS //Opposite of desaturation, makes everything more vibrant and saturated.

#include "/lib/otherDims/nether/colorConstants.glsl"

varying vec3 BLOCK_LIGHT_COLOR; //Color of block light. Gets yellow-er if you stay away from light-emitting blocks for a while.
varying vec4 HELD_LIGHT_COLOR; //Color of held light source. Alpha = brightness.

#include "/lib/noisetex.glsl"

/*
because this has to be defined in the .fsh stage in order for optifine to recognize it:
uniform float centerDepthSmooth;

const float eyeBrightnessHalflife = 20.0;
const float centerDepthHalflife = 1.0;

const int gaux3Format = RGBA16;
const int gcolorFormat = RGBA16;
const int compositeFormat = RGBA16;
const int gnormalFormat = RGB16;
*/

#include "/lib/math.glsl"

#include "/lib/otherDims/nether/calcMainLightColor.glsl"

#include "/lib/otherDims/nether/calcFogColor.glsl"

void main() {
	vec2 tc = texcoord;

	vec3 pos1 = vec3(tc, texture2D(depthtex1, tc).r);
	bool sky = pos1.z == 1.0;
	vec4 v1 = gbufferProjectionInverse * vec4(pos1 * 2.0 - 1.0, 1.0);
	pos1 = v1.xyz / v1.w;
	float dist1 = length(pos1) / far;

	vec3 color = texture2D(gcolor, tc).rgb;

	#ifdef FOG
		vec3 fogclr = calcFogColor(normalize((gbufferModelViewInverse * vec4(pos1, 1.0)).xyz));
	#else
		vec3 fogclr = fogColor;
	#endif

	if (sky) {
		color = fogclr * (1.0 - blindness);
	}
	else {
		vec4 aux = texture2D(gaux1, tc);
		float blocklight = aux.r;
		float heldlight = 0.0;

		color *= calcMainLightColor(blocklight, heldlight, dist1);

		vec2 lmcoord = aux.rg;

		#include "/lib/otherDims/nether/crossprocess.glsl"

		#ifdef FOG
			color = mix(fogclr, color, fogify(dist1, 0.25));
		#endif

		if (blindness > 0.0) color *= interpolateSmooth1(max(1.0 - dist1 * far * 0.2, 0.0)) * 0.5 * blindness + (1.0 - blindness);
	}

/* DRAWBUFFERS:0 */
	gl_FragData[0] = vec4(color, texture2D(gaux3, texcoord).r); //gcolor
}