#version 120

uniform float centerDepthSmooth;
uniform float far;
uniform float frameTimeCounter;
uniform int heldBlockLightValue;
uniform int heldItemId;
uniform ivec2 eyeBrightnessSmooth;
uniform mat4 gbufferProjection;
uniform mat4 gbufferProjectionInverse;

varying float eyeAdjust; //How much brighter to make the world

varying vec2 texcoord;

#define BLUR_ENABLED //Is blur enabled at all?
//#define DOF_BLUR //Blurs things that are at a different distance than whatever's in the center of your screen

#define DYNAMIC_LIGHTS //Holding blocks that emit light will light up their surroundings
#define EYE_ADJUST //Allows your eyes to "adjust" to darkness

varying vec3 BLOCK_LIGHT_COLOR; //Color of block light. Gets yellow-er if you stay away from light-emitting blocks for a while.
varying vec4 HELD_LIGHT_COLOR; //Color of held light source. Alpha = brightness.

#if defined(BLUR_ENABLED) && defined(DOF_BLUR)
	varying float dofDistance; //Un-projected centerDepthSmooth
#endif

#include "/lib/noisetex.glsl"

#include "/lib/calcHeldLightColor.glsl"

void main() {
	gl_Position = ftransform();
	texcoord = (gl_TextureMatrix[0] * gl_MultiTexCoord0).xy;

	#ifdef EYE_ADJUST
		eyeAdjust = 2.5 - eyeBrightnessSmooth.x / 240.0;
	#else
		eyeAdjust = 1.5;
	#endif

	#if defined(BLUR_ENABLED) && defined(DOF_BLUR)
		vec4 v = gbufferProjectionInverse * vec4(0.0, 0.0, centerDepthSmooth * 2.0 - 1.0, 1.0);
		dofDistance = -v.z / v.w;
	#endif

	#include "/lib/otherDims/nether/colors.glsl"

	#include "/lib/heldlightData.glsl"
}