#version 120

uniform float viewWidth;
uniform sampler2D gaux3;

varying vec2 texcoord;

#define BLUR_ENABLED //Is blur enabled at all?
#define BLUR_QUALITY 10 //Number of sample points to use for blurring. Higher quality = higher performance impact! [5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25]

float pixelSize = 1.0 / viewWidth;

#include "/lib/math.glsl"

void main() {
	#ifdef BLUR_ENABLED
		vec3 color = vec3(0.0);
		vec4 data = texture2D(gaux3, texcoord);
		data.a = 1.0 - data.a;
		if (data.a > 0.0) {
			float radius = data.a * 256.0;
			float total = 0.0;
			float start  = max(texcoord.x - radius * pixelSize,       pixelSize * 0.5);
			float finish = min(texcoord.x + radius * pixelSize, 1.0 - pixelSize * 0.5);
			float step   = max(pixelSize * 0.5, radius * pixelSize / float(BLUR_QUALITY));

			for (float x = start; x <= finish; x += step) {
				float weight = fogify(((texcoord.x - x) * viewWidth) / radius, 0.35);
				vec4 newdata = texture2D(gaux3, vec2(x, texcoord.y));
				newdata.a = 1.0 - newdata.a;
				weight *= newdata.a / data.a;
				color += newdata.rgb * newdata.rgb * weight;
				total += weight;
			}
			color = sqrt(color / total);
		}
		else color = texture2D(gaux3, texcoord).rgb;
	#else
		vec3 color = texture2D(gaux3, texcoord).rgb;
	#endif

/* DRAWBUFFERS:3 */
	gl_FragData[0] = vec4(color, texture2D(gaux3, texcoord).a); //gcolor
}