#version 120

uniform float far;
uniform float frameTimeCounter;
uniform int heldBlockLightValue;
uniform int heldItemId;
uniform int heldItemId2;
uniform ivec2 eyeBrightnessSmooth;
uniform mat4 gbufferProjection;

varying float mcentity; //ID data of block currently being rendered.
varying vec2 lmcoord;
varying vec2 texcoord;
varying vec3 normal;
varying vec4 glcolor;

varying vec3 BLOCK_LIGHT_COLOR; //Color of block light. Gets yellow-er if you stay away from light-emitting blocks for a while.
varying vec4 HELD_LIGHT_COLOR; //Color of held light source. Alpha = brightness.

#define DYNAMIC_LIGHTS //Holding blocks that emit light will light up their surroundings
#define IDLE_HANDS //Makes your hands sway back and forth in 1st person, like they do in 3rd person

#ifdef IDLE_HANDS
	vec2 handOffset = vec2(sin(frameTimeCounter * 1.6), sin(frameTimeCounter * 1.2)) / 64.0;
#endif

#include "/lib/noisetex.glsl"

#include "/lib/calcHeldLightColor.glsl"

void main() {
	texcoord = (gl_TextureMatrix[0] * gl_MultiTexCoord0).xy;
	lmcoord = (gl_TextureMatrix[1] * gl_MultiTexCoord1).xy;

	#ifdef IDLE_HANDS
		vec4 pos = gl_ModelViewMatrix * gl_Vertex;
		pos.xy += handOffset * sign(gl_ModelViewMatrix[3][0] + 0.3125);
		gl_Position = gl_ProjectionMatrix * pos;
	#else
		gl_Position = ftransform();
	#endif

	normal = normalize(gl_Normal) * 0.5 + 0.5;
	glcolor = gl_Color;
	glcolor.rgb *= min(normalize(gl_NormalMatrix * gl_Normal).y * 0.375 + 0.625 + heldBlockLightValue / 30.0, 1.25);

	#include "/lib/otherDims/nether/colors.glsl"

	#include "/lib/heldlightData.glsl"

	int heldItem = gl_ModelViewMatrix[3][0] > -0.3125 ? heldItemId : heldItemId2;
	if (heldItem == 95 || heldItem == 160) mcentity = 2.1; //stained glass
	else if (heldItem == 79) mcentity = 4.1; //ice
	else mcentity = 0.0;
}