#version 120

attribute vec3 mc_Entity;

uniform float far;
uniform float frameTimeCounter;
uniform int heldBlockLightValue;
uniform int heldItemId;
uniform ivec2 eyeBrightnessSmooth;
uniform mat4 gbufferModelViewInverse;
uniform mat4 gbufferProjection;

varying float mcentity; //ID data of block currently being rendered.
varying vec2 lmcoord;
varying vec2 texcoord;
varying vec3 normal;
varying vec3 pos;
varying vec4 glcolor;

varying vec3 BLOCK_LIGHT_COLOR; //Color of block light. Gets yellow-er if you stay away from light-emitting blocks for a while.
varying vec4 HELD_LIGHT_COLOR; //Color of held light source. Alpha = brightness.

#define DYNAMIC_LIGHTS //Holding blocks that emit light will light up their surroundings
//#define ID_FIX //Enable this if "effect blocks" are lacking their respective effects. For example if grass doesn't sway back and forth, or if water doesn't refract things behind it.

#include "/lib/noisetex.glsl"

#include "/lib/calcHeldLightColor.glsl"

void main() {
	texcoord = (gl_TextureMatrix[0] * gl_MultiTexCoord0).xy;
	lmcoord = (gl_TextureMatrix[1] * gl_MultiTexCoord1).xy;
	glcolor = gl_Color;
	gl_Position = ftransform();
	pos = (gbufferModelViewInverse * (gl_ModelViewMatrix * gl_Vertex)).xyz;
	pos.y -= 1.6;
	normal = normalize(gl_Normal) * 0.5 + 0.5;

	//Using IDs above 10000 to represent all blocks that I care about
	//if the ID is less than 10000, then I don't need to do extra logic to see if it has special effects.
	#ifndef ID_FIX
		if (mc_Entity.x > 10000.0) {
			int id = int(mc_Entity.x) - 10000;
	#else
			int id = int(mc_Entity.x);
	#endif
			if      (id == 9)  mcentity = 1.1; //water
			else if (id == 95) mcentity = 2.1; //stained glass
			else if (id == 79) mcentity = 3.1; //ice
			else mcentity = 0.1;
	#ifndef ID_FIX
		}
		else mcentity = 0.1;
	#endif

	#include "/lib/otherDims/nether/colors.glsl"

	#include "/lib/heldlightData.glsl"
}