#version 120

uniform float blindness;
uniform float far;
uniform float nightVision;
uniform float rainStrength;
uniform float screenBrightness;
uniform float viewHeight;
uniform float viewWidth;
uniform float wetness;
uniform int isEyeInWater;
uniform ivec2 eyeBrightness;
uniform ivec2 eyeBrightnessSmooth;
uniform mat4 gbufferModelViewInverse;
uniform mat4 gbufferProjectionInverse;
uniform sampler2D depthtex0;
uniform sampler2D depthtex1;
uniform sampler2D gaux1;
uniform sampler2D gaux2;
uniform sampler2D gaux3;
uniform sampler2D gaux4; //lightmap
#define lightmap gaux4
uniform sampler2D gcolor;
uniform sampler2D gnormal;
uniform vec3 cameraPosition;
        vec3 eyePosition = cameraPosition + gbufferModelViewInverse[3].xyz; //because cameraPosition isn't actually the position of the camera -_-
uniform vec3 fogColor;
uniform vec3 skyColor;

varying float adjustedTime; //More useful number than worldTime.
varying float day; //1.0 during the day, 0.0 at night.
varying float night; //1.0 at night, 0.0 during the day.
varying float sunset; //1.0 at sunset (when day and night are both < 1.0).

varying vec2 texcoord;
varying vec3 sunPosNorm;
varying vec3 upPosNorm;

#define FOG //It's fog, what more is there to say here?
#define UNDERWATER_FOG //Applies fog to water
#define DYNAMIC_LIGHTS //Holding blocks that emit light will light up their surroundings
//#define VANILLA_LIGHTMAP //Uses vanilla light colors instead of custom ones. Requires optifine 1.12.2 HD_U_D1 or later!
#define VIGNETTE //Reduces the brightness of dynamic light around edges the of your screen
#define DESATURATE //De-saturates the world at night, during rain, and in the end
//#define CROSS_PROCESS //Opposite of desaturation, makes everything more vibrant and saturated.
#define BRIGHT_WATER //Overrides light levels under water to be higher
#define INFINITE_OCEANS //Simulates water out to the horizon instead of just your render distance.
//#define CUBIC_CHUNKS //Disables black fog/sky colors below Y=0

#define SUNSET_COEFFICIENT_RED 7.2 //Red sunset coefficient. Higher values will result in the red color of sunset starting earlier and persisting longer. A change of 0.1 corresponds to about 5 seconds real-world time. [6.0 6.1 6.2 6.3 6.4 6.5 6.6 6.7 6.8 6.9 7.0 7.1 7.2 7.3 7.4 7.5 7.6 7.7 7.8 7.9 8.0]
#define SUNSET_COEFFICIENT_GREEN 6.8 //Green sunset coefficient. Higher values will result in the green color of sunset starting earlier and persisting longer. A change of 0.1 corresponds to about 5 seconds real-world time. [6.0 6.1 6.2 6.3 6.4 6.5 6.6 6.7 6.8 6.9 7.0 7.1 7.2 7.3 7.4 7.5 7.6 7.7 7.8 7.9 8.0]
#define SUNSET_COEFFICIENT_BLUE 6.2 //Blue sunset coefficient. Higher values will result in the blue color of sunset starting earlier and persisting longer. A change of 0.1 corresponds to about 5 seconds real-world time. [6.0 6.1 6.2 6.3 6.4 6.5 6.6 6.7 6.8 6.9 7.0 7.1 7.2 7.3 7.4 7.5 7.6 7.7 7.8 7.9 8.0]

#include "/lib/constants.glsl"

varying vec3 BLOCK_LIGHT_COLOR; //Color of block light. Gets yellow-er if you stay away from light-emitting blocks for a while.
varying vec3 SHADOW_COLOR; //Color of shadows. Sky-colored, to simulate indirect lighting.
varying vec3 SKY_LIGHT_COLOR; //Color of sky light. Is usually white during the day, and very dark blue at night.
varying vec4 HELD_LIGHT_COLOR; //Color of held light source. Alpha = brightness.

bool inWater = isEyeInWater == 1; //quicker to type.

/*
because this has to be defined in the .fsh stage in order for optifine to recognize it:
uniform float centerDepthSmooth;

const float eyeBrightnessHalflife = 20.0;
const float wetnessHalflife = 100.0;
const float drynessHalflife = 60.0;
const float centerDepthHalflife = 1.0;

const int gcolorFormat = RGBA16;
const int compositeFormat = RGBA16;
const int gaux3Format = RGBA16;
const int gnormalFormat = RGB16;
*/

#include "/lib/math.glsl"

#include "/lib/calcMainLightColor.glsl"

#include "/lib/calcFogColor.glsl"

#include "/lib/calcUnderwaterFogColor.glsl"

void main() {
	vec2 tc = texcoord;

	vec3 pos = vec3(tc, texture2D(depthtex0, tc).r);
	bool nothingInFrontOfSky = pos.z == 1.0;
	vec4 v = gbufferProjectionInverse * vec4(pos * 2.0 - 1.0, 1.0);
	pos = v.xyz / v.w;
	float dist = length(pos);
	vec3 posNorm = pos / dist;
	dist /= far;

	vec3 pos1 = vec3(tc, texture2D(depthtex1, tc).r);
	bool sky = pos1.z == 1.0;
	vec4 v1 = gbufferProjectionInverse * vec4(pos1 * 2.0 - 1.0, 1.0);
	pos1 = v1.xyz / v1.w;
	float dist1 = length(pos1) / far;

	vec3 color = texture2D(gcolor, tc).rgb;
	vec4 aux = texture2D(gaux1, tc);

	vec4 aux2 = texture2D(gaux2, tc);
	vec4 normal = texture2D(gnormal, tc);
	normal.xyz = normal.xyz * 2.0 - 1.0;
	bool water = int(aux2.b * 10.0 + 0.1) == 1; //only ID I'm actually checking for in this stage.

	float underwaterEyeBrightness = eyeBrightnessSmooth.y / 240.0;
	#ifdef BRIGHT_WATER
		underwaterEyeBrightness = underwaterEyeBrightness * 0.5 + 0.5;
	#endif

	if (!sky) {
		float skylight = aux.g;
		float blocklight = aux.r;
		float heldlight = 0.0;

		#ifdef BRIGHT_WATER
			if      ( water && !inWater) skylight = mix(skylight, skylight * 0.5 + 0.5, aux2.g); //max(skylight, aux2.g * 0.5);
			else if (!water &&  inWater) skylight = skylight * 0.5 + 0.5;
		#endif

		color *= calcMainLightColor(blocklight, skylight, heldlight, dist1);

		vec2 lmcoord = aux.rg;

		#include "/lib/desaturate.glsl"

		#include "/lib/crossprocess.glsl"

		//!water && !inWater = white fog in stage 1
		//!water &&  inWater = blue fog
		// water && !inWater = blue fog in stage 1 then white fog in stage 2
		// water &&  inWater = white fog in stage 1 then blue fog in stage 2

		//if water xor  inwater then blue fog
		//if water ==   inwater then white fog (stage 1)
		//if water and  inwater then blue fog
		//if water and !inwater then white fog (stage 2)

		#ifdef UNDERWATER_FOG
			if      (water && !inWater) color = calcUnderwaterFogColor(color, dist1 - dist, aux2.g * aux2.g);
			else if (!water && inWater) color = calcUnderwaterFogColor(color, dist1, underwaterEyeBrightness);
		#endif

		#ifdef FOG
			if (water == inWater) {
				float d = water ? dist1 - dist : dist1;
				d += wetness * eyeBrightnessSmooth.y * 0.00125 - 0.2; //wetness * 0.3 * eyeBrightness / 240.0 - 0.2
				if (d > 0.0) {
					float y = (gbufferModelViewInverse * vec4(pos1, 0.0)).y + eyePosition.y;
					d = fogify(d * (rainStrength + 1.0) * exp2(1.5 - y * 0.015625), 0.25);
					float actualEyeBrightness = eyeBrightness.y / 240.0;
					#ifdef BRIGHT_WATER
						if (inWater) actualEyeBrightness = actualEyeBrightness * 0.5 + 0.5;
					#endif
					color = mix(calcFogColor(posNorm) * min(max(aux.g, actualEyeBrightness) * 2.0, 1.0), color, d);
				}
			}
		#endif

		if (blindness > 0.0) color.rgb *= interpolateSmooth1(max(1.0 - dist1 * far * 0.2, 0.0)) * 0.5 * blindness + (1.0 - blindness);
	}
	else {
		aux2.g = 1.0;
		if (eyePosition.y < SEA_LEVEL) {
			if (inWater) {
				#ifdef INFINITE_OCEANS
					if (aux.a < 0.02) color = aux2.a < 0.02 ? calcUnderwaterFogColorInfinity(underwaterEyeBrightness) : calcFogColor(posNorm);
					else {
						if (aux2.a < 0.02) {
							aux2.b = 0.1;
							normal = vec4(0.0, -1.0, 0.0, 1.0);
							water = true;
						}
					}
				#else
					if (nothingInFrontOfSky) color = calcUnderwaterFogColorInfinity(underwaterEyeBrightness);
				#endif
			}
			#ifdef INFINITE_OCEANS
				else if (aux.a < 0.02) color = calcFogColor(posNorm);
			#endif
		}
		else {
			#ifdef INFINITE_OCEANS
				if (aux.a < 0.02) { //bottom half of sky
					if (aux2.a < 0.02) {
						color = calcUnderwaterFogColorInfinity(0.96875);
						aux2.b = 0.1;
						normal = vec4(0.0, 1.0, 0.0, 1.0);
						water = true;
					}
					else color = water && !inWater ? calcUnderwaterFogColorInfinity(aux2.g * aux2.g) : calcFogColor(posNorm);
				}
			#else
				if (water && !inWater) color = calcUnderwaterFogColorInfinity(aux2.g * aux2.g);
			#endif
		}

		color *= 1.0 - blindness;
	}

/* DRAWBUFFERS:025 */
	gl_FragData[0] = vec4(color, texture2D(gaux3, texcoord).r); //gcolor, storing transparency data in alpha channel
	gl_FragData[1] = normal * 0.5 + 0.5; //gnormal
	gl_FragData[2] = aux2; //gaux2
}