#version 120

uniform float viewHeight;
uniform sampler2D composite;

varying vec2 texcoord;

#define BLUR_ENABLED //Is blur enabled at all?
#define BLUR_QUALITY 10 //Number of sample points to use for blurring. Higher quality = higher performance impact! [5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25]

float pixelSize = 1.0 / viewHeight;

#include "/lib/math.glsl"

void main() {
	//float loops = 0.0;
	#ifdef BLUR_ENABLED
		vec3 color = vec3(0.0);
		vec4 data = texture2D(composite, texcoord);
		data.a = 1.0 - data.a;
		if (data.a > 0.0) {
			float radius = data.a * 256.0;
			float total = 0.0;
			float start  = max(texcoord.y - radius * pixelSize,       pixelSize * 0.5);
			float finish = min(texcoord.y + radius * pixelSize, 1.0 - pixelSize * 0.5);
			float step   = max(pixelSize * 0.5, radius * pixelSize / float(BLUR_QUALITY));

			for (float y = start; y <= finish; y += step) {
				float weight = fogify(((texcoord.y - y) * viewHeight) / radius, 0.35);
				vec4 newdata = texture2D(composite, vec2(texcoord.x, y));
				newdata.a = 1.0 - newdata.a;
				weight *= newdata.a / data.a;
				color += newdata.rgb * newdata.rgb * weight;
				total += weight;
				//loops += 1.0;
			}
			color = sqrt(color / total);
		}
		else color = texture2D(composite, texcoord).rgb;
	#else
		vec3 color = texture2D(composite, texcoord).rgb;
	#endif

/* DRAWBUFFERS:0 */
	gl_FragData[0] = vec4(color, 1.0); //gcolor
	//gl_FragData[0] = vec4(vec3(loops / 64.0), 1.0); //gcolor
}