#version 120

uniform float frameTimeCounter;
uniform int blockEntityId;
uniform mat4 gbufferModelViewInverse;
uniform sampler2D gaux1; //Overworld texture
uniform sampler2D gaux2; //End island texture
uniform sampler2D texture;

varying vec2 lmcoord;
varying vec2 texcoord;
varying vec3 pos;
varying vec4 glcolor;

#define ENDER_NEBULAE //Adds animated nebulae to the background of the end dimension
#define ENDER_ARCS //Adds bolts of plasma that arc through the nebulae. Requires ender nebulae to be enabled!
#define ENDER_STARS //Adds blinking stars to the background of the end dimension. Stackable with nebulae/plasma.
#define END_PORTAL_EFFECTS_OVERWORLD //Enables fancy effects for end portals
#define END_PORTAL_BACKGROUND_OVERWORLD 2 //1: Use overworld fog color. 2: Use end background. [1 2]
#define END_PORTAL_FOREGROUND_OVERWORLD 2 //0: No foreground image. 1: Use overworld screenshot. 2: Use end island screenshot. [0 1 2]
#define END_PORTAL_CLOUDS_OVERWORLD 2 //0: No clouds. 1: Use overworld clouds. 2: Use void clouds. [0 1 2]
//#define BRIGHT_PORTAL_FIX //Enable this if end portals are 16x brighter than they should be
//#define OLD_CLOUDS //Uses old cloud rendering method from earlier versions, for people who don't like pretty things.

#include "/lib/noisetex.glsl"

#include "/lib/math.glsl"

#ifdef END_PORTAL_EFFECTS_OVERWORLD
	#include "/lib/goldenOffsets.glsl"

	#include "/lib/hue.glsl"

	#if END_PORTAL_BACKGROUND_OVERWORLD == 2
		#include "/lib/endEffects.glsl"
	#endif

	#if END_PORTAL_CLOUDS_OVERWORLD == 1
		#ifdef OLD_CLOUDS
			#include "/lib/otherDims/end/fastDrawClouds_old.glsl"
		#else
			#include "/lib/otherDims/end/fastDrawClouds.glsl"
		#endif
	#elif END_PORTAL_CLOUDS_OVERWORLD == 2
		#include "/lib/fastDrawVoidClouds.glsl"
	#endif
#endif

void main() {
	vec4 color = texture2D(texture, texcoord) * glcolor;

	#ifdef END_PORTAL_EFFECTS_OVERWORLD
		if (blockEntityId == 119) {
			vec3 worldPos = (gbufferModelViewInverse * vec4(pos, 1.0)).xyz;

			#include "/lib/endPortalEffects.glsl"
		}
	#endif

/* DRAWBUFFERS:04 */
	gl_FragData[0] = color; //gcolor
	gl_FragData[1] = vec4(lmcoord, 1.0, 1.0); //gaux1
}