#version 120

uniform float far;
uniform float frameTimeCounter;
uniform float rainStrength;
uniform int heldBlockLightValue;
uniform int heldItemId;
uniform int heldItemId2;
uniform int moonPhase;
uniform int worldTime;
uniform mat4 gbufferProjection;
uniform ivec2 eyeBrightnessSmooth;
uniform vec3 fogColor;
uniform vec3 skyColor;
uniform vec3 sunPosition;

varying float day; //1.0 during the day, 0.0 at night.
varying float night; //1.0 at night, 0.0 during the day.
varying float id; //ID data of block currently being rendered.

varying vec2 lmcoord;
varying vec2 texcoord;
varying vec3 normal;
varying vec4 glcolor;

vec3 sunPosNorm = normalize(sunPosition);

varying vec3 BLOCK_LIGHT_COLOR; //Color of block light. Gets yellow-er if you stay away from light-emitting blocks for a while.
varying vec3 SHADOW_COLOR; //Color of shadows. Sky-colored, to simulate indirect lighting.
varying vec3 SKY_LIGHT_COLOR; //Color of sky light. Is usually white during the day, and very dark blue at night.
varying vec4 HELD_LIGHT_COLOR; //Color of held light source. Alpha = brightness.

#define DYNAMIC_LIGHTS //Holding blocks that emit light will light up their surroundings
#define HARDCORE_DARKNESS 0 //0 (Off): Normal visibility at night. 1 (On): Complete darkness at night. 2 (Moon phase) Nighttime brightness is determined by the current phase of the moon. [0 1 2]
#define IDLE_HANDS //Makes your hands sway back and forth in 1st person, like they do in 3rd person

#define SUNSET_COEFFICIENT_RED 7.2 //Red sunset coefficient. Higher values will result in the red color of sunset starting earlier and persisting longer. A change of 0.1 corresponds to about 5 seconds real-world time. [6.0 6.1 6.2 6.3 6.4 6.5 6.6 6.7 6.8 6.9 7.0 7.1 7.2 7.3 7.4 7.5 7.6 7.7 7.8 7.9 8.0]
#define SUNSET_COEFFICIENT_GREEN 6.8 //Green sunset coefficient. Higher values will result in the green color of sunset starting earlier and persisting longer. A change of 0.1 corresponds to about 5 seconds real-world time. [6.0 6.1 6.2 6.3 6.4 6.5 6.6 6.7 6.8 6.9 7.0 7.1 7.2 7.3 7.4 7.5 7.6 7.7 7.8 7.9 8.0]
#define SUNSET_COEFFICIENT_BLUE 6.2 //Blue sunset coefficient. Higher values will result in the blue color of sunset starting earlier and persisting longer. A change of 0.1 corresponds to about 5 seconds real-world time. [6.0 6.1 6.2 6.3 6.4 6.5 6.6 6.7 6.8 6.9 7.0 7.1 7.2 7.3 7.4 7.5 7.6 7.7 7.8 7.9 8.0]

#include "/lib/noisetex.glsl"

#include "/lib/calcHeldLightColor.glsl"

#ifdef IDLE_HANDS
	vec2 handOffset = vec2(sin(frameTimeCounter * 1.6), sin(frameTimeCounter * 1.2)) / 64.0;
#endif

void main() {
	texcoord = (gl_TextureMatrix[0] * gl_MultiTexCoord0).xy;
	lmcoord = (gl_TextureMatrix[1] * gl_MultiTexCoord1).xy;

	#ifdef IDLE_HANDS
		vec4 pos = gl_ModelViewMatrix * gl_Vertex;
		pos.xy += handOffset * sign(gl_ModelViewMatrix[3][0] + 0.3125);
		gl_Position = gl_ProjectionMatrix * pos;
	#else
		gl_Position = ftransform();
	#endif

	glcolor = gl_Color;
	#include "/lib/heldlightData.glsl"

	int heldItem = gl_ModelViewMatrix[3][0] > -0.3125 ? heldItemId : heldItemId2;
	if (heldItem == 95 || heldItem == 160) id = 0.2; //stained glass
	else if (heldItem == 79) id = 0.4; //ice
	else id = 0.0;

	float adjustedTime = abs(mod(worldTime / 1000.0 + 6.0, 24.0) - 12.0);
	day = clamp(6.7 - adjustedTime, 0.0, 1.0);
	night = clamp(adjustedTime - 6.7, 0.0, 1.0);
	float sunset = 1.0 - day - night;
	#if HARDCORE_DARKNESS == 2
		float phase = abs(float(moonPhase) * 0.25 - 1.0);
	#endif

	normal = normalize(gl_NormalMatrix * gl_Normal);
	#include "/lib/glmult.glsl"
	normal = normalize(gl_Normal) * 0.5 + 0.5;

	#include "/lib/colors.glsl"
}