#version 120

attribute vec3 mc_Entity;
attribute vec2 mc_midTexCoord;

uniform float frameTimeCounter;
uniform float rainStrength;
uniform float wetness;
uniform int worldTime;
uniform mat4 gbufferModelView;
uniform mat4 gbufferModelViewInverse;
uniform vec3 cameraPosition;
uniform vec3 sunPosition;

varying float ao;
varying float isDirt;
varying float isLava;
varying vec2 randCoord;

varying vec2 lmcoord;
varying vec2 texcoord;
varying vec4 glcolor;

//#define REMOVE_Y_OFFSET //Removes random Y offset from tallgrass/flowers/etc...
//#define REMOVE_XZ_OFFSET //Removes random X/Z offset from tallgrass/flowers/etc...
#define GRASS_AO //Adds ambient occlusion to tallgrass/flowers/etc... Works best with "Remove Y Offset" enabled.
#define GRASS_PATCHES //Makes grass less uniform by making patches of it dryer or lusher. Does not affect leaves.
#define HUMIDITY_OFFSET 1.1 //Higher number = lusher grass. Lower number = dryer grass [0.5 0.55 0.6 0.65 0.7 0.75 0.8 0.85 0.9 0.95 1.0 1.05 1.1 1.15 1.2 1.25]
#define WAVING_GRASS //Adds wind effects to grass
#define LAVA_PATCHES //Randomizes lava brightness, similar to grass patches
#define WET_DIRT //Hydrated hummus. Soggy soil. Drenched dirt. I can't think of a good name for this config option, but it makes dirt darker during rain to simulate being wet.
//#define LEGACY_SUGARCANE //Removes biome coloring from sugar cane

//#define ID_FIX //Enable this if "effect blocks" are lacking their respective effects. For example if grass doesn't sway back and forth, or if water doesn't refract things behind it.

float adjustedTime = abs(mod(worldTime / 1000.0 + 6.0, 24.0) - 12.0);
float night = clamp(adjustedTime - 6.7, 0.0, 1.0);
vec3 sunPosNorm = normalize(sunPosition);

#include "/lib/noisetex.glsl"

#include "/lib/math.glsl"

#ifdef WAVING_GRASS
	float approxScaledCos(float x) { //x from 0 to 1, y from -1 to 1
		x = abs(fract(x) * 2.0 - 1.0);
		return x * x * (6.0 - 4.0 * x) - 1.0;
		//x = fract(x);
		//if (x <= 0.5) return -16.0 * x * x + 8.0 * x;
		//else return 16.0 * x * x - 24.0 * x + 8.0;
	}

	vec3 windOffset(vec3 pos, float multiplier, float speed) {
		float baseWindAmt = min(rainStrength, wetness) * 1.5 + (pos.y / 192.0 + 0.66666666) * (1.0 - night * 0.5); //1.0x at y=64, 2.0x at y=256, and rain increases this to 2.5x at y=64 and 3.5x at y=256. Rain also increases it by 1.5x, and night decreases it to 0.5x.
		vec3 waveStart = texture2D(noisetex, vec2(pos.x + frameTimeCounter, pos.z) * 0.375 * invNoiseRes).rgb; //oscillation direction and phase offset
		float waveMultiplier = texture2D(noisetex, vec2(pos.x * 0.125 + frameTimeCounter * 0.5, pos.z * 0.125) * invNoiseRes).r * 0.5 + 0.5; //multiplier to add variety
		vec2 offset = vec2(waveStart.y * 0.4 - 0.2, waveStart.z * 0.2 - 0.1) * approxScaledCos(waveStart.x + frameTimeCounter * speed) * waveMultiplier; //combine to get position offset
		offset.x -= baseWindAmt * 0.01 + 0.02; //biased towards east wind
		offset *= multiplier * baseWindAmt; //scale offset
		return vec3(offset.x, 0.5 / (lengthSquared2(offset) + 0.5) - 1.0, offset.y); //move vertexes down some based on how much they were offset
	}
#endif

#ifdef GRASS_PATCHES
	float noiseMap(vec2 coord) {
		coord *= invNoiseRes;
		float noise = 0.0;
		noise += texture2D(noisetex, coord * 0.03125).r;
		noise += texture2D(noisetex, coord * 0.09375).r * 0.5;
		noise += texture2D(noisetex, coord * 0.375  ).r * 0.25;
		return noise;
	}
#endif

void main() {
	ao = 1.0;
	isLava = 0.0;
	isDirt = 0.0;

	bool isGrass = false;

	texcoord = (gl_TextureMatrix[0] * gl_MultiTexCoord0).xy;
	lmcoord = (gl_TextureMatrix[1] * gl_MultiTexCoord1).xy;
	vec4 pos = gbufferModelViewInverse * (gl_ModelViewMatrix * gl_Vertex); //chunk coords -> world coords

	glcolor = gl_Color;

	vec3 normal;
	//Using IDs above 10000 to represent all blocks that I care about
	//if the ID is less than 10000, then I don't need to do extra logic to see if it has special effects.
	#ifndef ID_FIX
		if (mc_Entity.x > 10000.0) {
			int id = int(mc_Entity.x) - 10000;
	#else
			int id = int(mc_Entity.x);
	#endif

			if (id == 2) { //grass
				normal = normalize(gl_NormalMatrix * gl_Normal);

				#ifdef GRASS_PATCHES
					isGrass = gl_Color.g > gl_Color.b;
				#endif

				#ifdef WET_DIRT
					if (abs(gl_Color.g - gl_Color.b) < 0.02) isDirt = 1.0;
				#endif
			}
			else if (id == 31 || id == 175) { //plants and double plants
				normal = normalize(gl_NormalMatrix * vec3(0.0, 1.0, 0.0));

				#ifdef REMOVE_Y_OFFSET
					pos.y = floor(pos.y + cameraPosition.y + 0.5) - cameraPosition.y;
				#endif
				#ifdef REMOVE_XZ_OFFSET
					pos.xz = floor(pos.xz + cameraPosition.xz + 0.5) - cameraPosition.xz;
				#endif

				#if defined(GRASS_AO) || defined(WAVING_GRASS)
					#ifdef WAVING_GRASS
						float windSpeed = 0.75;
					#endif

					float amt = float(texcoord.y < mc_midTexCoord.y);
					if (id == 175) {
						amt = (amt + float(mc_Entity.z >= 8.0)) * 0.5;
						#ifdef GRASS_AO
							ao = amt;
						#endif

						#ifdef WAVING_GRASS
							windSpeed = 0.5;
							amt *= 1.5;
						#endif
					}
					#ifdef GRASS_AO
						else ao = amt;
					#endif

					#ifdef WAVING_GRASS
						if (amt > 0.1) { //will always either be 0.0, 0.5 or 1.0
							vec3 offset = windOffset(pos.xyz + cameraPosition, amt * lmcoord.y * lmcoord.y, windSpeed);
							pos.xyz += offset;
						}
					#endif
				#endif

				#ifdef GRASS_PATCHES
					isGrass = gl_Color.g > gl_Color.b;
				#endif
			}
			else if (id == 59) { //crops
				normal = normalize(gl_NormalMatrix * vec3(0.0, 1.0, 0.0));

				#ifdef GRASS_AO
					ao = float(texcoord.y < mc_midTexCoord.y);
				#endif

				#ifdef WAVING_GRASS
					if (texcoord.y < mc_midTexCoord.y) {
						vec3 offset = windOffset(pos.xyz + cameraPosition, lmcoord.y * lmcoord.y, 0.75);
						pos.xyz += offset;
					}
				#endif
			}
			else if (id == 83) { //sugar cane and other arbitrarily-tall plants
				normal = normalize(gl_NormalMatrix * vec3(0.0, 1.0, 0.0));

				#ifdef LEGACY_SUGARCANE
					glcolor = vec4(1.0);
				#endif

				#ifdef GRASS_PATCHES
					isGrass = true;
				#endif
			}

			#ifdef LAVA_PATCHES
				else if (id == 11) { //lava
					normal = normalize(gl_NormalMatrix * gl_Normal);
					vec3 worldPos = pos.xyz + cameraPosition;
					isLava = 1.0;
					if (abs(gl_Normal.y) > 0.1) randCoord = worldPos.xz * 2.0;
					else randCoord = vec2((worldPos.x + worldPos.z) * 4.0, worldPos.y + frameTimeCounter);
				}
			#endif

			else if (id == 30) { //cobwebs and other stuff that shouldn't have shadows
				normal = normalize(gl_NormalMatrix * vec3(0.0, 1.0, 0.0));
			}
			else {
				normal = normalize(gl_NormalMatrix * gl_Normal);
			}
	#ifndef ID_FIX
		}
		else {
			normal = normalize(gl_NormalMatrix * gl_Normal);
		}
	#endif

	#ifdef GRASS_PATCHES
		if (isGrass) {
			float noise = noiseMap(pos.xz + cameraPosition.xz) - HUMIDITY_OFFSET;
			noise = (noise - wetness * 0.125) * (wetness * -0.5 + 1.0); //more lush with less variation during rain
			if (noise > 0.0) glcolor.rg += vec2(noise * 0.33333333, noise * -0.125);
			else glcolor.rb += noise * 0.25;
			glcolor.g = max(glcolor.g, glcolor.r * 0.85);
		}
	#endif

	#include "/lib/glmult.glsl"

	gl_Position = gl_ProjectionMatrix * (gbufferModelView * pos);
}