#version 120

uniform float blindness;
uniform float centerDepthSmooth;
uniform float far;
uniform float frameTimeCounter;
uniform int heldBlockLightValue;
uniform int heldItemId;
uniform ivec2 eyeBrightnessSmooth;
uniform mat4 gbufferModelViewInverse;
uniform mat4 gbufferProjection;
uniform mat4 gbufferProjectionInverse;
uniform vec3 cameraPosition;
        vec3 eyePosition = cameraPosition + vec3(0.0, gbufferModelViewInverse[3][1], 0.0);

varying vec2 texcoord;

#define BLUR_ENABLED //Is blur enabled at all?
//#define DOF_BLUR //Blurs things that are at a different distance than whatever's in the center of your screen

#define DYNAMIC_LIGHTS //Holding blocks that emit light will light up their surroundings
#define VOID_CLOUDS //Dark ominous clouds in the end
#define VOID_CLOUD_HEIGHT 128.0 //Y level of void clouds [-64.0 -48.0 -32.0 -16.0 0.0 16.0 32.0 48.0 64.0 80.0 96.0 112.0 128.0 144.0 160.0 176.0 192.0 208.0 224.0 240.0 256.0 272.0 288.0 304.0 320.0 336.0 352.0 368.0 384.0 400.0 416.0 432.0 448.0 464.0 480.0 496.0 512.0]

varying vec3 BLOCK_LIGHT_COLOR; //Color of block light. Gets yellow-er if you stay away from light-emitting blocks for a while.
varying vec4 HELD_LIGHT_COLOR; //Color of held light source. Alpha = brightness.

#ifdef VOID_CLOUDS
	varying vec4 VOID_CLOUD_INSIDE_COLOR; //Color to render over your entire screen when inside a void cloud.
#endif

#if defined(BLUR_ENABLED) && defined(DOF_BLUR)
	varying float dofDistance; //Un-projected centerDepthSmooth
#endif

#include "/lib/noisetex.glsl"

#include "/lib/math.glsl"

#include "/lib/goldenOffsets.glsl"

#include "/lib/calcHeldLightColor.glsl"

#ifdef VOID_CLOUDS
	#include "/lib/hue.glsl"

	#include "/lib/otherDims/end/drawVoidClouds.glsl"
#endif

void main() {
	gl_Position = ftransform();
	texcoord = (gl_TextureMatrix[0] * gl_MultiTexCoord0).xy;

	#include "/lib/otherDims/end/colors.glsl"

	#include "/lib/heldlightData.glsl"

	#if defined(BLUR_ENABLED) && defined(DOF_BLUR)
		vec4 v = gbufferProjectionInverse * vec4(0.0, 0.0, centerDepthSmooth * 2.0 - 1.0, 1.0);
		dofDistance = -v.z / v.w;
	#endif

	#ifdef VOID_CLOUDS
		float d = abs(eyePosition.y - VOID_CLOUD_HEIGHT) / 4.0;
		if (d < 1.0) {
			VOID_CLOUD_INSIDE_COLOR = drawVoidClouds(vec3(0.0, 1.0, 0.0), d);
			if (VOID_CLOUD_INSIDE_COLOR.a > 0.001) {
				if (d > 0.001 && d < 0.999) { //in the fadeout range
					//approximated cosine interpolation
					VOID_CLOUD_INSIDE_COLOR.a *= interpolateSmooth1(d);
					//if (d <= 0.5) VOID_CLOUD_INSIDE_COLOR.a *= 2.0 * d * d;
					//else VOID_CLOUD_INSIDE_COLOR.a *= -2.0 * d * d + 4.0 * d - 1.0;
					VOID_CLOUD_INSIDE_COLOR *= 0.9; //why am I doing this again?
				}
			}
		}
		else VOID_CLOUD_INSIDE_COLOR = vec4(0.0);
	#endif
}