#version 120

uniform float aspectRatio;
uniform float blindness;
uniform float far;
uniform float frameTimeCounter;
uniform float nightVision;
uniform float screenBrightness;
uniform float viewHeight;
uniform float viewWidth;
uniform int isEyeInWater;
uniform ivec2 eyeBrightness;
uniform ivec2 eyeBrightnessSmooth;
uniform mat4 gbufferModelView;
uniform mat4 gbufferModelViewInverse;
uniform mat4 gbufferProjectionInverse;
uniform sampler2D composite;
uniform sampler2D depthtex0;
uniform sampler2D gaux2;
uniform sampler2D gaux4;
#define lightmap gaux4
uniform sampler2D gcolor;
uniform sampler2D gnormal;
uniform vec3 cameraPosition;
        vec3 eyePosition = cameraPosition + gbufferModelViewInverse[3].xyz; //because cameraPosition isn't actually the position of the camera -_-
uniform vec3 fogColor;
uniform vec3 skyColor;

varying float eyeAdjust; //How much brighter to make the world

varying vec2 texcoord;

#define BLUR_ENABLED //Is blur enabled at all?
#define UNDERWATER_BLUR //Blurs the world while underwater
#define WATER_BLUR //Blurs things behind water
#define GLASS_BLUR //Blurs things behind stained glass
#define ICE_BLUR //Blurs things behind ice
//#define DOF_BLUR //Blurs things that are at a different distance than whatever's in the center of your screen

#define FOG //It's fog, what more is there to say here?
#define UNDERWATER_FOG //Applies fog to water
#define DYNAMIC_LIGHTS //Holding blocks that emit light will light up their surroundings
//#define VANILLA_LIGHTMAP //Uses vanilla light colors instead of custom ones. Requires optifine 1.12.2 HD_U_D1 or later!
#define VIGNETTE //Reduces the brightness of dynamic light around edges the of your screen
//#define CROSS_PROCESS //Opposite of desaturation, makes everything more vibrant and saturated.
#define WATER_REFRACT //Adds "waves" to water that distort things behind them
#define REFLECT //Reflects the sun/sky onto reflective surfaces. Does not add reflections of terrain!
#define ICE_SCATTER //Scatters things behind ice
#define ALT_GLASS //Uses alternate blending method for stained glass which looks more like real stained glass
#define BRIGHT_WATER //Overrides light levels under water to be higher
//#define CUBIC_CHUNKS //Disables black fog/sky colors below Y=0

#if defined(BLUR_ENABLED) && defined(DOF_BLUR)
	varying float dofDistance; //Un-projected centerDepthSmooth
#endif

varying vec4 HELD_LIGHT_COLOR; //Color of held light source. Alpha = brightness.

#include "/lib/noisetex.glsl"

#include "/lib/math.glsl"

/*
#ifdef BLUR_ENABLED
#endif
#ifdef UNDERWATER_BLUR
#endif
#ifdef WATER_BLUR
#endif
#ifdef GLASS_BLUR
#endif
#ifdef ICE_BLUR
#endif
#ifdef RAIN_BLUR
#endif
#ifdef DOF_BLUR
#endif

Are you happy now, optifine?
*/

//basically the simplified mathematical form of "clr * mix(multiplier, 1.0, clr)".
//the brighter a color was originally, the less of an effect multiplier has on it.
vec3 brightenColor(vec3 clr, float multiplier) {
	return clr * (multiplier - clr * (multiplier - 1.0));
}

#include "/lib/goldenOffsets.glsl"

#include "/lib/otherDims/twilight_forest/calcMainLightColor.glsl"

#include "/lib/noiseLOD.glsl"

#include "/lib/otherDims/twilight_forest/calcFogColor.glsl"

#include "/lib/otherDims/twilight_forest/calcUnderwaterFogColor.glsl"

void main() {
	#include "/lib/lavaOverlay.glsl"

	vec2 tc = texcoord;

	vec3 oldAux2 = texture2D(gaux2, texcoord).rgb;
	int id = int(oldAux2.b * 10.0 + 0.1);
	vec3 normal = texture2D(gnormal, texcoord).xyz * 2.0 - 1.0;

	vec3 pos = vec3(texcoord, texture2D(depthtex0, texcoord).r);
	bool nothingInFrontOfSky = pos.z == 1.0;
	vec4 v = gbufferProjectionInverse * vec4(pos * 2.0 - 1.0, 1.0);
	pos = v.xyz / v.w;

	float dist = length(pos);
	vec3 posNorm = pos / dist;

	vec3 worldPos = (gbufferModelViewInverse * vec4(pos, 1.0)).xyz + cameraPosition;

	#ifdef REFLECT
		float reflective = 0.0;
	#endif

	float blur = 0.0;

	#if defined(BLUR_ENABLED) && defined(DOF_BLUR)
		blur = interpolateSmooth1(min(abs(dist - dofDistance) / dofDistance, 1.0)) * 4.0;
	#endif

	#if defined(BLUR_ENABLED) && defined(WATER_BLUR)
		float waterBlur = 4.0; //slightly more dynamic than other types of blur
	#endif

	if (id == 1) { //water
		#ifdef REFLECT
			reflective = 0.5;
		#endif

		#ifdef WATER_REFRACT
			vec3 newPos = worldPos;
			ivec2 swizzles;
			float multiplier = 1.0;
			if (abs(normal.y) > 0.1) { //top/bottom surfaces
				if (abs(normal.y) < 0.999) newPos.xz -= normalize(normal.xz) * (frameTimeCounter * 3.0);
				else multiplier = oldAux2.g * 0.75 + 0.25;
				swizzles = ivec2(0, 2);
			}
			else {
				newPos.y += frameTimeCounter * 4.0;
				if (abs(normal.x) < 0.02) swizzles = ivec2(0, 1);
				else swizzles = ivec2(2, 1);
			}

			vec2 offset = waterNoiseLOD(vec2(newPos[swizzles[0]], newPos[swizzles[1]]), dist) * (multiplier * 0.015625);
			normal[swizzles[0]] += offset[0] * 4.0;
			normal[swizzles[1]] += offset[1] * 4.0;

			offset.y *= aspectRatio;

			tc += offset / max(dist * 0.0625, 1.0);
		#endif
	}
	else if (id == 2) { //stained glass
		#ifdef REFLECT
			reflective = 0.25;
		#endif

		#if defined(BLUR_ENABLED) && defined(GLASS_BLUR)
			blur = max(blur, 8.0);
		#endif
	}
	else if (id == 3 || id == 4) { //ice and held ice
		#ifdef REFLECT
			reflective = 0.25;
		#endif

		#if defined(BLUR_ENABLED) && defined(ICE_BLUR)
			blur = max(blur, 4.0);
		#endif

		#ifdef ICE_SCATTER
			vec3 offset;
			if (id == 3) { //normal ice
				vec2 coord = (abs(normal.y) < 0.001 ? vec2(worldPos.x + worldPos.z, worldPos.y) : worldPos.xz);
				offset = iceNoiseLOD(coord * 256.0, dist) * 0.0078125;
			}
			else { //held ice
				vec2 coord = gl_FragCoord.xy + 0.5;
				offset = iceNoise(coord * 0.5) * 0.0078125;
			}

			normal += offset * 8.0;

			offset.y *= aspectRatio;
			vec2 newtc = tc + offset.xy;
			vec3 newnormal = texture2D(gnormal, newtc).xyz * 2.0 - 1.0;
			if (dot(normal, newnormal) > 0.9) tc = newtc; //don't offset on the edges of ice
		#endif
	}

	vec3 aux2 = texture2D(gaux2, tc).rgb;
	if (abs(aux2.b - oldAux2.b) > 0.02) {
		tc = texcoord;
		aux2 = texture2D(gaux2, tc).rgb;
	}

	vec4 c = texture2D(gcolor, tc);
	vec3 color = c.rgb;
	float transparentAlpha = c.a; //using gcolor to store composite's alpha
	vec4 transparent = texture2D(composite, tc); //transparency of closest object to the camera

	#if defined(BLUR_ENABLED) && defined(UNDERWATER_BLUR)
		if (isEyeInWater == 1) blur = 6.0;
	#endif

	dist /= far;

	if (transparentAlpha > 0.001) {
		#ifdef ALT_GLASS
			if (id == 2) {
				vec3 transColor = transparent.rgb / transparentAlpha;
				color *= min(transColor * 2.0, 1.0); //because the default colors are too dark to be used.

				float skylight = aux2.g;
				float blocklight = aux2.r;
				float heldlight = 0.0;

				color += transColor * calcMainLightColor(blocklight, skylight, heldlight, dist) * 0.125 * (1.0 - blindness);
			}
			else {
		#endif
			color = mix(color, transparent.rgb / transparentAlpha, transparentAlpha);
		#ifdef ALT_GLASS
			}
		#endif
	}

	#ifdef REFLECT
		reflective *= aux2.g * aux2.g * (1.0 - blindness);
		vec3 reflectedPos;
		if (isEyeInWater == 0 && reflective > 0.001) { //sky reflections
			vec3 newnormal = normalize((gbufferModelView * vec4(normal, 0.0)).xyz);
			reflectedPos = reflect(posNorm, newnormal);
			vec3 skyclr = calcFogColor(reflectedPos);
			float posDot = dot(-posNorm, newnormal);
			color += skyclr * square(square(1.0 - max(posDot, 0.0))) * reflective;
		}
	#endif

	if (id >= 1) { //everything that I've currently assigned effects to so far needs fog to be done in this stage.
		if (isEyeInWater == 1) {
			#ifdef UNDERWATER_FOG
				float actualEyeBrightness = eyeBrightnessSmooth.y / 240.0;
				#ifdef BRIGHT_WATER
					actualEyeBrightness = actualEyeBrightness * 0.5 + 0.5;
				#endif
				color = calcUnderwaterFogColor(color, dist, actualEyeBrightness) * (1.0 - blindness);
			#endif
		}
		else {
			#ifdef FOG
				float d = dist - 0.2;
				if (d > 0.0) {
					d = fogify(d * exp2(1.5 - worldPos.y * 0.015625), 0.25);
					color = mix(calcFogColor(posNorm) * min(max(aux2.g * 2.0, eyeBrightness.y / 120.0), 1.0) * (1.0 - blindness), color, d);
					#if defined(BLUR_ENABLED) && defined(WATER_BLUR)
						waterBlur *= d;
					#endif
				}
			#endif
		}
	}

	color = min(color, 1.0); //reflections (And possibly other things) can go above maximum brightness

	#if defined(BLUR_ENABLED) && defined(WATER_BLUR)
		if (id == 1 && isEyeInWater == 0) blur += waterBlur;
	#endif

	#ifdef BLUR_ENABLED
		blur /= 256.0;
	#endif

	color = brightenColor(color, eyeAdjust);

/* DRAWBUFFERS:6 */
	gl_FragData[0] = vec4(color, 1.0 - blur); //gcolor
}