#version 120

uniform float far;
uniform float frameTimeCounter;
uniform int heldBlockLightValue;
uniform int heldItemId;
uniform int heldItemId2;
uniform mat4 gbufferProjection;

varying float id; //ID data of block currently being rendered.

varying vec2 lmcoord;
varying vec2 texcoord;
varying vec3 normal;
varying vec4 glcolor;

varying vec4 HELD_LIGHT_COLOR; //Color of held light source. Alpha = brightness.

#define DYNAMIC_LIGHTS //Holding blocks that emit light will light up their surroundings
#define IDLE_HANDS //Makes your hands sway back and forth in 1st person, like they do in 3rd person

#include "/lib/noisetex.glsl"

#include "/lib/calcHeldLightColor.glsl"

#ifdef IDLE_HANDS
	vec2 handOffset = vec2(sin(frameTimeCounter * 1.6), sin(frameTimeCounter * 1.2)) / 64.0;
#endif

void main() {
	texcoord = (gl_TextureMatrix[0] * gl_MultiTexCoord0).xy;
	lmcoord = (gl_TextureMatrix[1] * gl_MultiTexCoord1).xy;

	#ifdef IDLE_HANDS
		vec4 pos = gl_ModelViewMatrix * gl_Vertex;
		pos.xy += handOffset * sign(gl_ModelViewMatrix[3][0] + 0.3125);
		gl_Position = gl_ProjectionMatrix * pos;
	#else
		gl_Position = ftransform();
	#endif

	glcolor = gl_Color;
	#include "/lib/heldlightData.glsl"

	int heldItem = gl_ModelViewMatrix[3][0] > -0.3125 ? heldItemId : heldItemId2;
	if (heldItem == 95 || heldItem == 160) id = 0.2; //stained glass
	else if (heldItem == 79) id = 0.4; //ice
	else id = 0.0;

	normal = normalize(gl_Normal) * 0.5 + 0.5;
}