#version 120

uniform float blindness;
uniform float far;
uniform float nightVision;
uniform float screenBrightness;
uniform float viewHeight;
uniform float viewWidth;
uniform int isEyeInWater;
uniform ivec2 eyeBrightness;
uniform ivec2 eyeBrightnessSmooth;
uniform mat4 gbufferModelView;
uniform mat4 gbufferModelViewInverse;
uniform mat4 gbufferProjectionInverse;
uniform sampler2D lightmap;
uniform sampler2D texture;
uniform vec3 cameraPosition;
        vec3 eyePosition = cameraPosition + gbufferModelViewInverse[3].xyz; //because cameraPosition isn't actually the position of the camera -_-
uniform vec3 fogColor;
uniform vec3 skyColor;

varying vec2 lmcoord;
varying vec2 texcoord;
varying vec3 normal;
varying vec4 glcolor;

varying float mcentity; //ID data of block currently being rendered.

#define FOG //It's fog, what more is there to say here?
#define DYNAMIC_LIGHTS //Holding blocks that emit light will light up their surroundings
//#define VANILLA_LIGHTMAP //Uses vanilla light colors instead of custom ones. Requires optifine 1.12.2 HD_U_D1 or later!
#define VIGNETTE //Reduces the brightness of dynamic light around edges the of your screen
//#define CROSS_PROCESS //Opposite of desaturation, makes everything more vibrant and saturated.
#define CLEAR_WATER //Overwrites water texture to be completely transparent
#define BRIGHT_WATER //Overrides light levels under water to be higher
#define ALT_GLASS //Uses alternate blending method for stained glass which looks more like real stained glass
//#define CUBIC_CHUNKS //Disables black fog/sky colors below Y=0

varying vec4 HELD_LIGHT_COLOR; //Color of held light source. Alpha = brightness.

#include "/lib/math.glsl"

#include "/lib/otherDims/twilight_forest/calcMainLightColor.glsl"

#include "/lib/otherDims/twilight_forest/calcFogColor.glsl"

void main() {
	int id = int(mcentity);
	vec4 color = texture2D(texture, texcoord) * glcolor;

	#ifdef CLEAR_WATER
		if (id == 1) {
			color.a = 0.0;
		}
		else {
	#endif
		vec3 pos = vec3(gl_FragCoord.xy / vec2(viewWidth, viewHeight), gl_FragCoord.z);
		vec4 v = gbufferProjectionInverse * vec4(pos * 2.0 - 1.0, 1.0);
		pos = v.xyz / v.w;

		float dist = length(pos) / far;

		float skylight = lmcoord.y;
		float blocklight = lmcoord.x;
		float heldlight = 0.0;

		#ifdef BRIGHT_WATER
			if (isEyeInWater == 1) skylight = skylight * 0.5 + 0.5;
		#endif

		#ifdef ALT_GLASS
			bool applyEffects = true;
		#endif

		if (id == 2) { //stained glass
			if (color.a > 0.6) {
				color.a = 1.0; //1.0 - (1.0 - color.a) * 0.5; //make borders more opaque
				id = 0;
			}
			#ifdef ALT_GLASS
				else applyEffects = false; //don't apply lighting effects to the center of glass when ALT_GLASS is enabled, since composite1 takes care of that instead.
			#endif
		}

		#ifdef ALT_GLASS
			if (applyEffects) {
		#endif
				color.rgb *= calcMainLightColor(blocklight, skylight, heldlight, dist);

				#include "/lib/crossprocess.glsl"

				if (blindness > 0.0) color.rgb *= interpolateSmooth1(max(1.0 - dist * far * 0.2, 0.0)) * 0.5 * blindness + (1.0 - blindness);

		#ifdef ALT_GLASS
			}
		#endif

		#ifdef FOG
			#ifdef ALT_GLASS //don't apply fog to glass when better glass is enabled, since this is done in composite1 instead
				if (id != 2) {
			#endif
				#ifndef CLEAR_WATER //already checked at the beginning if clear water is enabled, only needs to be re-checked if it's not.
					if (id != 1) { //water fog is handled in composite and composite1, to be compatible with infinite oceans
				#endif
						float d = dist - 0.2;
						if (d > 0.0) {
							float y = (gbufferModelViewInverse * vec4(pos, 1.0)).y + cameraPosition.y;
							d = fogify(d * exp2(1.5 - y * 0.015625), 0.25);
							color.rgb = mix(calcFogColor(pos / (dist * far)) * min(max(lmcoord.y * 2.0, eyeBrightness.y / 120.0), 1.0) * (1.0 - blindness), color.rgb, d); //min(max(aux2.g, eyeBrightness.y), 1.0)
						}
				#ifndef CLEAR_WATER
					}
				#endif
			#ifdef ALT_GLASS
				}
			#endif
		#endif
	#ifdef CLEAR_WATER
		}
	#endif

/* DRAWBUFFERS:2563 */
	gl_FragData[0] = vec4(normal, 1.0); //gnormal
	gl_FragData[1] = vec4(lmcoord, id * 0.1, 1.0); //gaux2
	gl_FragData[2] = vec4(1.0, 0.0, 0.0, color.a); //gaux3
	gl_FragData[3] = color; //composite
}